from seatools.ioc.injects import Autowired
from typing import TypeVar, Type, Any
from fastapi import Depends


_T = TypeVar("_T", bound=Any)


def from_bean(value: str = None, *, cls: Type[_T] = None, required: bool = True) -> _T:
    """
    适用于将seatools.ioc容器转换到FastAPI的依赖注入
    Args:
        value: seatools.ioc bean name
        cls:  seatools.ioc bean type
        required: 是否必须

    Returns:
        FastAPI 的 Depends 依赖注入对象, 注入的实际对象为 seatools.ioc 的容器实例
    """
    async def depends_bean() -> _T:
        return Autowired(value, cls=cls, required=required)

    return Depends(depends_bean)


FastAutowired = from_bean



from sqlalchemy import text
from sqlalchemy.orm import Session

from seatools.ioc import Bean, run
from seatools.ioc.database import DatabaseConfig
from seatools.sqlalchemy import SqlAlchemyClient
from seatools.ioc.sqlalchemy import new_session


def test_ioc_database():
    cfg = DatabaseConfig(
        host='localhost',
        port=6379,
        password='123456',
        database=0,
        driver='redis'
    )
    print(cfg.render_to_string())
    print(cfg.render_to_string(False))


@Bean
def sqlalchemy_client() -> SqlAlchemyClient:
    return SqlAlchemyClient(
        url='sqlite:///test.db'
    )


@new_session(autocommit=False)
def handler(session: Session):
    ans = session.execute(text('select 2')).scalars().first()
    assert ans == 2


def test_sqlalchemy():
    run(scan_package_names=['tests.test_ioc_database'])

    handler()



from .base import BaseBeanProxy, ClassBeanProxy, AspectClassBeanProxy, AsyncCallClassBeanProxy
from .delay import DelayAutowiredClassBeanProxy, AsyncCallDelayAutowiredClassBeanProxy, DelayConfigAutowiredClassBeanProxy
